/*
 * debug.c
 *
 * Purpose:
 * --------
 * Routines de debuggage
 * En particulier gestion d'un log ds debug ds une fenetre
 *
 * History:
 * --------
 * 25.09.94: fplanque: Created
 */

 
 	#include "!OPTIONS.H"				/* Options de compilation */         
	#define	THIS_FILE	"DEBUG.C v1.00 - 03.95"

/*
 * System headers:
 */
	#include	<stdio.h>
	#include <string.h>   

/*
 * Custom headers:
 */
	#include	"SPEC_PU.H"
	#include	"WIN_PU.H"
	#include	"TEXT_PU.H"
	#include	"DEBUG_PU.H"	




/*
 * ------------------------- MEMBRES PUBLICS ----------------------------
 */
	WIPARAMS	*	G_debug_window = NULL;
	char 			Tsz_DebugMsg[256];

/*
 * ------------------------- MEMBRES PRIVES ----------------------------
 */
	/*
	 * Texte log en ram:
	 */
	TEXTINFO *	M_pTextInfo_Debug;



/*
 * ----------------------- OPERATIONS PRIVEES -------------------------
 */
	/*
	 * Cration du log texte en RAM
	 * appell par constructeur uniquement
	 */
	void	debug_CreateRamText( void );

 
/*
 * ------------------------ IMPLEMENTATION -------------------------
 */




/*
 * debug_Construct(-)
 *
 * Purpose:
 * --------
 * Construction de "l'Objet" debug
 * init des variables etc..
 *
 * History:
 * --------
 * 25.09.94: fplanque: Created
 */
void	debug_Construct( void )
{
	/*
	 * Cration du log de debuggage en RAM:
	 * (invisible pour l'instant)
	 */
	debug_CreateRamText();
}



/*
 * create_debugtext(-)
 *
 * Purpose:
 * --------
 * Cration de l'en-tte du texte de debug
 * ainsi que de ses paramtres d'affichage
 *
 * History:
 * --------
 * 1993: fplanque: Created
 * 25.09.94: devient fonction indpendante, la mise en fentre s'effectue sparment
 */
void	debug_CreateRamText( void )
{
	/*
	 * Cre un texte:
	 */
	M_pTextInfo_Debug = create_TextInfo();

	/*
	 * Cre quelques lignes 
	 */
	text_addLineToEnd( M_pTextInfo_Debug, "Stut One version 3" );
	text_addLineToEnd( M_pTextInfo_Debug, "Copyright (c)1993-94 by Franois PLANQUE." );
	text_addLineToEnd( M_pTextInfo_Debug, NULL );
	text_addLineToEnd( M_pTextInfo_Debug, "Ready." );

}




/*
 * debug_CreateWinText(-)
 *
 * Purpose:
 * --------
 * Prparation du texte de debug pour son affichage ne fenetre
 * ainsi que de ses paramtres d'affichage
 *
 * History:
 * --------
 * 1993: fplanque: Created
 * 25.05.94: fplanque: Le paramtre est dsormais un TEXT_DRAWPAR
 * 25.09.94: le texte n'est plus cr sur place, il a dj t cr avant
 */
int	debug_CreateWinText( 
			TEXT_DRAWPAR * *	ppTextDrawPar )
{
	/*
	 * Cre des paramtres d'affichages pour ce texte:
	 */
	*ppTextDrawPar = create_TextDrawPar( M_pTextInfo_Debug, 200 );

	if ( *ppTextDrawPar == NULL )
	{
		return	WRONG0;
	}

	return	OK1;
}



/*
 * signale(-)
 *
 * Purpose:
 * --------
 * Signal + Information de debbugging dans la fentre debug
 * ou a dfaut ds le log invisible
 *
 * History:
 * --------
 * 1993: fplanque: Created
 * 25.09.94: fplanque: appelle maintenant debug_AddLine()
 */
void	signale( 
			const char * inf )		/* In: texte  dupliker et insrer ds Log */
{
	ping();
	debug_AddLine( inf );
}



/*
 * debug_AddLine(-)
 *
 * Purpose:
 * --------
 * Information de debbugging dans la fentre debug
 * ou a dfaut ds le log invisible
 *
 * History:
 * --------
 * 25.09.94: extracted from signale()
 * 25.09.94: ajoute au log invisible si pas de fen ouverte
 */
void	debug_AddLine( 
			const char * inf )		/* In: texte  dupliker et insrer ds Log */
{
	if ( G_debug_window != NULL )
	{	/*
		 * S'il existe une fentre de debug: 
		 */
		add_textinf( G_debug_window, inf );
	}
	else
	{	/*
		 * Pas de fentre debug:
		 * on ajoute qd mme au log invisible (pour l'instant)
		 */
		text_addLineToEnd( M_pTextInfo_Debug, inf );
	}
}
